const express = require('express');
const robot = require('./robot')
const {getRustConfig} = require('./config')
const http = require("./utils/http");
const file = require('./utils/file')
const {logger, fileLogger} = require("./utils/logger");
const {spawn, exec, fork, execSync} = require("child_process");
const path = require("path");

/*******
 * web
 * **** */

function init() {
    const app = express();
    const port = 3000;
    // 中间件示例，用于处理JSON请求体
    app.use(express.json());

    // GET请求的路由示例
    app.get('/', (req, res) => {
        res.send('is OK!');
    });


    //简单三个接口，为了方便使用get请求
    app.get('/isOK', isOK);
    app.get('/logs', (req, res) => {
        // 获取发送过来的信息（请求体的数据）
        const param = req.query;
        const n = param.n
        const id = param.id

        //时间
        const today = new Date();
        const year = today.getFullYear(); // 获取年份
        const month = (today.getMonth() + 1).toString().padStart(2, '0');
        const day = today.getDate().toString().padStart(2, '0');
        // 拼接字符串得到 YYYY-MM-DD 格式
        var logName = `${year}-${month}-${day}`;

        var f = getRustConfig().logPath + "/" + logName + ".log"
        var array = [];
        if (file.checkFilePath(f)) {
            const directoryPath = path.resolve(f);
            array = file.readLastNLines(directoryPath, n)
        } else {
            logger.info('日志文件不存在:' + f);
        }

        res.send({'code': 200, 'data': array, "message": "SUCCESS"});
    });
    app.post('/execute', (req, res) => {
        // 获取发送过来的信息（请求体的数据）
        const param = req.body;
        (async (param) => {
            console.log('请求体:', param);
            var executeType = param.executeType
            if (executeType === "RUN") {
                await robot.run(param)
            } else if (executeType === "STOP") {
                await robot.closeApp(param)
            } else if (executeType === "RESTART") {
                await robot.run(param)
            }
        })(param);
        // 回应信号（响应请求）
        res.send({'code': 200, 'data': "null", "message": "SUCCESS"});
    })

    // 使服务器监听特定端口
    app.listen(port, () => {
        // console.log(`服务器正在监听 http://localhost:${port}`);
    });

    logger.info('--web 启动');
}

// node-已上线-上报1
function robotNodeStatus() {
    var config = getRustConfig()
    http.request_post(`${config.baseUrl}/report/beOnline`, {},{...config.headers}).then((data) => {
        // logger.info('??', data);
        logger.info('node 在线-上报成功！',data);
    }).catch((error) => {
        logger.error(`node 在线！-上报失败: ${error.message}`); // 处理可能发生的错误
    });
}




function isOK(req, res) {
    res.send({'code': 200, 'data': "null", "message": "SUCCESS"});
}

// 日志读取操作
module.exports = {
    init,
    robotNodeStatus
};